/*
 * This file is part of queso.
 *
 * Copyright © 2020 Canek Peláez Valdés
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *    Canek Peláez Valdés <canek@ciencias.unam.mx>
 */

#pragma once

#include <linux/videodev2.h>
#include <stdbool.h>

typedef struct _V4LControlManager V4LControlManager;
typedef struct _V4LControl V4LControl;
typedef enum _V4LControlType V4LControlType;
typedef struct _V4LControlMenuItem V4LControlMenuItem;

enum _V4LControlType 
{
        V4L_CONTROL_TYPE_INT      = V4L2_CTRL_TYPE_INTEGER,
        V4L_CONTROL_TYPE_BOOLEAN  = V4L2_CTRL_TYPE_BOOLEAN,
        V4L_CONTROL_TYPE_MENU     = V4L2_CTRL_TYPE_MENU,
        V4L_CONTROL_TYPE_BUTTON   = V4L2_CTRL_TYPE_BUTTON,
        V4L_CONTROL_TYPE_INT64    = V4L2_CTRL_TYPE_INTEGER64,
        V4L_CONTROL_TYPE_STRING   = V4L2_CTRL_TYPE_STRING,
        V4L_CONTROL_TYPE_BITMASK  = V4L2_CTRL_TYPE_BITMASK,
        V4L_CONTROL_TYPE_INT_MENU = V4L2_CTRL_TYPE_INTEGER_MENU,
        V4L_CONTROL_TYPE_UINT8    = V4L2_CTRL_TYPE_U8,
        V4L_CONTROL_TYPE_UINT16   = V4L2_CTRL_TYPE_U16,
        V4L_CONTROL_TYPE_UINT32   = V4L2_CTRL_TYPE_U32,
        V4L_CONTROL_TYPE_INVALID  = -1
};

V4LControlManager*  v4l_control_manager_new            (const char*        device);
int                 v4l_control_manager_get_size       (V4LControlManager* v4lmgr);
V4LControl*         v4l_control_manager_nth_control    (V4LControlManager* v4lmgr,
                                                        int                n);
void                v4l_control_manager_free           (V4LControlManager* v4lmgr);

V4LControl*         v4l_control_new                    (V4LControlManager* v4lmgr,
                                                        int                id,
                                                        char*              name,
                                                        V4LControlType     type,
                                                        int64_t            minimum,
                                                        int64_t            maximum,
                                                        u_int64_t          step,
                                                        int64_t            default_value);
int                 v4l_control_get_id                 (V4LControl*        v4lctl);
char*               v4l_control_get_name               (V4LControl*        v4lctl);
void                v4l_control_free                   (V4LControl*        v4lctl);
V4LControlType      v4l_control_get_type               (V4LControl*        v4lctl);
int64_t             v4l_control_get_minimum            (V4LControl*        v4lctl);
int64_t             v4l_control_get_maximum            (V4LControl*        v4lctl);
u_int64_t           v4l_control_get_step               (V4LControl*        v4lctl);
int64_t             v4l_control_get_default_value      (V4LControl*        v4lctl);
int64_t             v4l_control_get_value              (V4LControl*        v4lctl);
void                v4l_control_set_value              (V4LControl*        v4lctl,
                                                        int64_t            value);
int                 v4l_control_get_menu_size          (V4LControl*        v4lctl);
V4LControlMenuItem* v4l_control_nth_menu_item          (V4LControl*        v4lctl,
                                                        int                n);

u_int32_t           v4l_control_menu_item_get_cid      (V4LControlMenuItem *v4lcmi);
u_int32_t           v4l_control_menu_item_get_index    (V4LControlMenuItem *v4lcmi);
const char*         v4l_control_menu_item_get_name     (V4LControlMenuItem *v4lcmi);
int64_t             v4l_control_menu_item_get_value    (V4LControlMenuItem *v4lcmi);
bool                v4l_control_menu_item_is_disabled  (V4LControlMenuItem *v4lcmi);
