/*
 * This file is part of queso.
 *
 * Copyright © 2020 Canek Peláez Valdés
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *    Canek Peláez Valdés <canek@ciencias.unam.mx>
 */

namespace Queso {

    /**
     * Class for framerates.
     */
    public class Framerate : GLib.Object, Gee.Comparable<Framerate> {

        /**
         * The numerator of the framerate.
         */
        public int numerator { public get; private set; }

        /**
         * The denominator of the framerate.
         */
        public int denominator { public get; private set; }

        /**
         * The framerate value.
         */
        public double value {
            public get {
                return (double)numerator / (double)denominator;
            }
        }

        /**
         * Constructs a framerate.
         */
        public Framerate(int numerator, int denominator)
        requires (numerator > 0 && denominator > 0) {
            this.numerator = numerator;
            this.denominator = denominator;
        }

        /**
         * Compares the two framerates.
         * @param fr the framerate to compare with.
         * @return -1 if this framerate it's better than the one received; 0 if
         *         they are equal; 1 if it's worse.
         */
        public int compare_to(Framerate fr) {
            if (value > fr.value)
                return -1;
            if (value < fr.value)
                return 1;
            return 0;
        }
    }
}
