/*
 * This file is part of queso.
 *
 * Copyright © 2020 Canek Peláez Valdés
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *    Canek Peláez Valdés <canek@ciencias.unam.mx>
 */

[CCode (cheader_filename = "uv4l.h")]
namespace Queso {

    [Compact]
    [CCode (cname = "V4LControlManager",
            construct_function = "v4l_control_manager_new",
            free_function = "v4l_control_manager_free")]
    public class V4LControlManager {
        [CCode (cname = "v4l_control_manager_new")]
        public V4LControlManager(string device);
        public int size {
            [CCode (cname = "v4l_control_manager_get_size")] get;
        }
        [CCode (cname = "v4l_control_manager_nth_control")]
        public unowned V4LControl nth_control(int nth);
    }

    [Compact]
    [CCode (cname = "V4LControl",
            construct_function = "v4l_control_new",
            free_function = "v4l_control_free")]
    public class V4LControl {
        public V4LControl(int id, string name);
        public int id {
            [CCode (cname = "v4l_control_get_id")] get;
        }
        public unowned string name {
            [CCode (cname = "v4l_control_get_name")] get;
        }
        public V4LControlType v4l_type {
            [CCode (cname = "v4l_control_get_type")] get;
        }
        public int64 minimum {
            [CCode (cname = "v4l_control_get_minimum")] get;
        }
        public int64 maximum {
            [CCode (cname = "v4l_control_get_maximum")] get;
        }
        public uint64 step {
            [CCode (cname = "v4l_control_get_step")] get;
        }
        public int64 default_value {
            [CCode (cname = "v4l_control_get_default_value")] get;
        }
        public int64 value {
            [CCode (cname = "v4l_control_get_value")] get;
            [CCode (cname = "v4l_control_set_value")] set;
        }
        public int64 menu_size {
            [CCode (cname = "v4l_control_get_menu_size")] get;
        }
        [CCode (cname = "v4l_control_nth_menu_item")]
        public unowned V4LControlMenuItem nth_menu_item(int n);
    }

    [Compact]
    [CCode (cname = "V4LControlMenuItem")]
    public class V4LControlMenuItem {
        public int cid {
            [CCode (cname = "v4l_control_menu_item_get_cid")] get;
        }
        public int index {
            [CCode (cname = "v4l_control_menu_item_get_index")] get;
        }
        public unowned string name {
            [CCode (cname = "v4l_control_menu_item_get_name")] get;
        }
        public int64 value {
            [CCode (cname = "v4l_control_menu_item_get_value")] get;
        }
        public bool disabled {
            [CCode (cname = "v4l_control_menu_item_is_disabled")] get;
        }
    }

    [CCode(cheader_filename = "uv4l.h", cname = "V4LControlType",
           cprefix = "V4L_CONTROL_TYPE_")]
    public enum V4LControlType {
        INT,
        BOOLEAN,
        MENU,
        BUTTON,
        INT64,
        STRING,
        BITMASK,
        INT_MENU,
        UINT8,
        UINT16,
        UINT32,
        INVALID;

        public string to_string() {
            switch (this) {
            case INT:      return "integer";
            case BOOLEAN:  return "boolean";
            case MENU:     return "menu";
            case BUTTON:   return "button";
            case INT64:    return "64 bit integer";
            case STRING:   return "string";
            case BITMASK:  return "bitmask";
            case INT_MENU: return "integer menu";
            case UINT8:    return "unsigned 8 bit integer";
            case UINT16:   return "unsigned 16 bit integer";
            case UINT32:   return "unsigned 32 bit integer";
            default:       return "invalid";
            }
        }
    }
}
